; ; -*- Mode: Emacs-Lisp -*-

(setq inhibit-startup-message t)

;(setq debug-on-error t)

;; Global key maps and such
(global-set-key [home]   'beginning-of-line)
(global-set-key [end]    'end-of-line)

;; Final newline handling
; Now done by ethan-wspace.
;(setq require-final-newline t)          ; Silently ensure newline at end of file

;; Add line numbers to mode line.
(setq line-number-mode t)
(setq column-number-mode t)

;; Show mark visually.
(setq-default transient-mark-mode t)

;; show parenthesis
(show-paren-mode t)

(setq load-path (append (list "~/.emacs.d/emacs-packages/lisp") load-path))
(setq scroll-bar-mode-explicit t)
(set-scroll-bar-mode `right)
(set-frame-height (selected-frame) 80)
(set-frame-width (selected-frame) 80)

;; ***************************************************************
;; Basic Modes and Editing
;; ***************************************************************
(setq default-major-mode 'text-mode)
; (add-hook 'text-mode-hook 'turn-on-auto-fill)

(add-to-list 'auto-mode-alist '("Wishlist" . text-mode))
(add-to-list 'auto-mode-alist '("README" . text-mode))

;; Customizations for all modes in CC Mode.
(defun my-c-mode-common-hook ()
  (c-set-style "bsd")
  (setq indent-tabs-mode nil)
  (setq c-tab-always-indent nil)
  (setq c-basic-offset 4)
  ;; (setq mode-name "C4")

  ;; we like auto-newline and hungry-delete
  (c-toggle-auto-hungry-state 1)
  ;; keybindings for all supported languages.  We can put these in
  ;; c-mode-base-map because c-mode-map, c++-mode-map, objc-mode-map,
  ;; java-mode-map, idl-mode-map, and pike-mode-map inherit from it.
  ;;(define-key c-mode-base-map "\C-m" 'newline-and-indent)
  (define-key c-mode-base-map "\C-m" 'c-context-line-break)
)
(add-hook 'c-mode-common-hook 'my-c-mode-common-hook)

;; Java mode customisation: c-mode-common-hook run first.
(defun my-java-mode-hook ()
  (c-set-style "java")
  (c-set-offset 'substatement-open 0)
  (setq c-comment-only-line-offset . 0)
)
(add-hook 'java-mode-hook 'my-java-mode-hook)

;;(defun my-jde-mode-hook ()
;;  (c-set-style "java")
;;  (c-set-offset 'substatement-open 0)
;;  (setq c-comment-only-line-offset . 0)
;;)
;;(add-hook 'jde-mode-hook 'my-jde-mode-hook)

;; Perl mode customisations
(defun my-perl-mode-hook ()
  (setq perl-tab-always-indent nil)
  (define-key perl-mode-map "\C-m" 'newline-and-indent)
)
(add-hook 'perl-mode-hook 'my-perl-mode-hook)

(defun my-cperl-mode-hook ()
  (setq cperl-tab-always-indent nil)
  (setq cperl-indent-level 4)
  (setq cperl-continued-statement-offset 0)
  ;; (setq mode-name "CPerl4")
  (define-key cperl-mode-map "\C-m" 'newline-and-indent)
)
(add-hook 'cperl-mode-hook 'my-cperl-mode-hook)

(defun linux-c-mode ()
  "C mode with adjusted defaults for use with the Linux kernel."
  (interactive)
  (c-mode)
  (c-set-style "linux")
  (setq mode-name "Linux/C")
  (setq c-basic-offset 8))

(add-to-list 'auto-mode-alist '("/linu[sx].*/.*\\.[ch]\\'" . linux-c-mode))

;; Synergy C++ mode. Ugh.
(defun synergy-c-mode ()
  "C mode with adjusted defaults for use with Synergy."
  (interactive)
  (c++-mode)
  (setq mode-name "Synergy/C++")
  (c-set-style "K&R")
  (setq indent-tabs-mode t)
  (setq tab-width 4)
  (setq c-basic-offset 4))
  
(add-to-list 'auto-mode-alist '("/synergy.*/.*\\.[ch]p*\\'" . synergy-c-mode))

;; ***************************************************************
;; Tab setup
;; ***************************************************************

(setq-default tab-width 8
              standard-indent 8
              indent-tabs-mode nil)

(defun my-tab4 ()
  "Set tab spacing to 4"
  (interactive)
  (setq tab-width 4)
  (setq-default tab-width 4)
  (redraw-display)
)

(defun my-tab8 ()
  "Set tab spacing to 8"
  (interactive)
  (setq tab-width 8)
  (setq-default tab-width 8)
  (redraw-display)
)

(defun my-indent2 ()
  "Set C indents to 2"
  (interactive)
  (setq c-basic-offset 2)
  (setq cperl-indent-level 2)
  (setq jam-indent-size 2)
  (redraw-display)
)

(defun my-indent4 ()
  "Set C indents to 4"
  (interactive)
  (setq c-basic-offset 4)
  (setq cperl-indent-level 4)
  (setq jam-indent-size 4)
  (redraw-display)
)

(defun my-indent8 ()
  "Set C indents to 8"
  (interactive)
  (setq c-basic-offset 8)
  (setq cperl-indent-level 8)
  (setq jam-indent-size 8)
  (redraw-display)
)

(defun to-unix-text ()
  "Set text format of the buffer to Unix"
  (interactive)
  (if running-xemacs
    (toggle-buffer-file-coding-system)
    (set-buffer-file-coding-system 'undecided-unix))
)

(defun to-dos-text ()
  "Set text format of the buffer to DOS"
  (interactive)
  (if running-xemacs
    (toggle-buffer-file-coding-system)
    (set-buffer-file-coding-system 'undecided-dos))
)

;; ***************************************************************
;; Whitespace
;; ***************************************************************

(require 'ethan-wspace)

;; ***************************************************************
;; Copyright dates
;; ***************************************************************

(add-hook 'before-save-hook 'copyright-update)

;; ***************************************************************
;; Emacs tab bar stuff
;; ***************************************************************

(require 'tabbar)

(set-face-attribute
     'tabbar-default nil
     :background "gray60")
(set-face-attribute
     'tabbar-unselected nil
     :background "gray85"
     :foreground "gray30"
     :box nil)
(set-face-attribute
     'tabbar-selected nil
     :background "#f2f2f6"
     :foreground "black"
     :box nil)
(set-face-attribute
     'tabbar-button nil
     :box '(:line-width 1 :color "gray72" :style released-button))
(set-face-attribute
     'tabbar-separator nil
     :height 0.7)

(tabbar-mode 1)

;; ***************************************************************
;; Jam stuff
;; ***************************************************************

(require 'jam-mode)

;; Customizations for Jam mode
(defun my-jam-mode-hook ()
  ; (define-key jam-mode-map "\C-m" 'newline-and-indent)
)
(add-hook 'jam-mode-hook 'my-jam-mode-hook)

;; ***************************************************************
;; IDM stuff
;; ***************************************************************

(require 'idm-mode)

;; Customizations for IDM mode
(defun my-idm-mode-hook ()
  ;;(define-key idm-mode-map "\C-m" 'newline-and-indent)
  (define-key idm-mode-map "\C-m" 'idm-electric-newline)
)
(add-hook 'idm-mode-hook 'my-idm-mode-hook)

;; ***************************************************************
;; ABC stuff
;; ***************************************************************

(require 'abc-mode)

;; ***************************************************************
;; AsciiDoc stuff
;; ***************************************************************

(require 'adoc-mode)

(defun my-adoc-mode-hook ()
  (interactive)
  (setq buffer-face-mode-face '(:family "DejaVu Sans Condensed" :height 90))
  (buffer-face-mode t)
)

(add-to-list 'auto-mode-alist (cons "\\.adoc\\'" 'adoc-mode))
(add-hook 'adoc-mode-hook 'my-adoc-mode-hook)

;; ***************************************************************
;; Generic stuff
;; ***************************************************************

(require 'generic-extras)

;; Function keys on UNIX box.
(define-key global-map [f1]             'to-unix-text)
(define-key global-map [f2]             'to-dos-text)
(define-key global-map [f3]             'revert-buffer)
(define-key global-map [f4]             'my-tab4)
(define-key global-map [f5]             'blink-matching-open)
(define-key global-map [f8]             'my-tab8)
(define-key global-map [f9]             'auto-fill-mode)
(define-key global-map [f10]            'goto-line)

;; (define-key global-map [C-#]           'goto-line)

(define-key global-map [S-f2]           'my-indent2)
(define-key global-map [S-f4]           'my-indent4)
(define-key global-map [S-f8]           'my-indent8)

;; ***************************************************************
;; Font Locking Setup
;; ***************************************************************

(setq font-lock-maximum-decoration t)

(global-font-lock-mode t)

;; Mercurial extras
(defun grep-project (s)
  (interactive "sSearch project for: ")
   (grep-find (concat "files=(`hg manifest`);grep -nH -e \"" s "\" ${files[@]/#/`hg root`/}"))
)
(global-set-key (quote [f7]) (quote grep-project))

;; ***************************************************************
;; Custom stuff
;; ***************************************************************

(custom-set-variables
 ;; custom-set-variables was added by Custom.
 ;; If you edit it by hand, you could mess it up, so be careful.
 ;; Your init file should contain only one such instance.
 ;; If there is more than one, they won't work right.
 '(browser-url-generic-program "/usr/local/bin/firefox")
 '(column-number-mode t)
 '(default-frame-alist (quote ((foreground-color . "Black") (background-color . "#DADADA") (vertical-scroll-bars . right) (tool-bar-lines . 1) (menu-bar-lines . 1) (right-fringe) (left-fringe) (width . 80) (height . 80))))
 '(delete-key-deletes-forward t)
 '(efs-use-passive-mode t)
 '(font-lock-mode t t (font-lock))
 '(global-ethan-wspace-mode t)
 '(gnus-save-newsrc-file nil)
 '(gnus-select-method (quote (nntp "news")))
 '(gnuserv-program (concat exec-directory "/gnuserv"))
 '(html-helper-address-string "jim@bear-cave.org.uk")
 '(html-helper-use-expert-menu t)
 '(jde-build-function (quote (jde-ant-build)))
 '(jde-check-version-flag t)
 '(jde-enable-abbrev-mode nil)
 '(jde-gen-cflow-enable nil)
 '(jde-gen-k&r nil)
 '(ps-bottom-margin 50)
 '(ps-font-size 5)
 '(ps-header-font-size 5)
 '(ps-header-offset 10)
 '(ps-header-title-font-size 6)
 '(ps-inter-column 15)
 '(ps-landscape-mode nil)
 '(ps-left-margin 30)
 '(ps-number-of-columns 2)
 '(ps-paper-type (quote a4))
 '(ps-print-color-p nil)
 '(ps-right-margin 30)
 '(ps-top-margin 20)
 '(ps-zebra-stripes nil)
 '(query-user-mail-address nil)
 '(recent-files-permanent-submenu t)
 '(sgml-markup-faces (quote ((start-tag . default) (end-tag . default) (comment . default) (pi . default) (sgml . default) (doctype . default) (entity . default) (shortref . default) (ignored . default) (ms-start . default) (ms-end . default))))
 '(sgml-set-face nil t)
 '(show-paren-mode t)
 '(user-mail-address "jim@bear-cave.org.uk"))

(custom-set-faces
 ;; custom-set-faces was added by Custom.
 ;; If you edit it by hand, you could mess it up, so be careful.
 ;; Your init file should contain only one such instance.
 ;; If there is more than one, they won't work right.
 '(default ((t (:inherit nil :stipple nil :background "#DADADA" :foreground "Black" :inverse-video nil :box nil :strike-through nil :overline nil :underline nil :slant normal :weight normal :height 98 :width normal :foundry "unknown" :family "Ubuntu Mono"))))
 '(font-lock-builtin-face ((((class color) (background light)) (:foreground "BlueViolet"))))
 '(font-lock-comment-face ((((class color) (background light)) (:foreground "green4"))))
 '(font-lock-constant-face ((((class color) (background light)) (:foreground "DarkRed"))))
 '(font-lock-function-name-face ((((class color) (background light)) (:foreground "Blue"))))
 '(font-lock-keyword-face ((((class color) (background light)) (:foreground "DarkBlue"))))
 '(font-lock-negation-char-face ((t (:foreground "saddle brown"))))
 '(font-lock-preprocessor-face ((((class color) (background light)) (:foreground "DarkCyan"))))
 '(font-lock-reference-face ((((class color) (background light)) (:foreground "OrangeRed"))))
 '(font-lock-string-face ((((class color) (background light)) (:foreground "Turquoise4"))))
 '(font-lock-type-face ((((class color) (background light)) (:foreground "Purple"))))
 '(font-lock-variable-name-face ((((class color) (background light)) (:foreground "Blue4"))))
 '(font-lock-warning-face ((((class color) (background light)) (:foreground "red2"))))
 '(fringe ((t (:background "#DADADA")))))

